/*********************************************************************
*    Project: MG82F5B32-DEMO
*			MG82F5B32 LQFP32_V10 EV Board (TH156A)
*			CpuCLK=12MHz, SysCLK=12MHz
*    Description:
*			PCA clock: 12MHz
*			CH0(P22),CH1(P33),CH2(P24),CH3(P34),CH4(P26),CH5(P35),PWM6(P20),PWM7(P21)
*			Edge,8bit 256
*			PWM Freq: PCAClk/(256)=12MHz/256=46.875KHz
*    Note:
*
*    Creat time:
*    Modify:
*    
*********************************************************************/
#define _MAIN_C

#include <Intrins.h>
#include <Absacc.h>

#include <Stdio.h>  // for printf

#include ".\include\REG_MG82F5Bxx.H"
#include ".\include\Type.h"
#include ".\include\API_Macro_MG82F5B32.H"
#include ".\include\API_Uart_BRGRL_MG82F5B32.H"

/*************************************************
*Set SysClk (MAX.50MHz)
*Selection:
*	11059200,12000000,
*	22118400,24000000,
*	29491200,32000000,
*	44236800,48000000
*************************************************/
#define MCU_SYSCLK		12000000
/*************************************************/
/*************************************************
*Set CpuClk (MAX.32MHz)
*	1) CpuCLK=SysCLK
*	2) CpuClk=SysClk/2
*************************************************/
#define MCU_CPUCLK		(MCU_SYSCLK)
//#define MCU_CPUCLK		(MCU_SYSCLK/2)

#define TIMER_1T_1ms_TH	((65536-(u16)(float)(1000*((float)(MCU_SYSCLK)/(float)(1000000)))) /256) 			
#define TIMER_1T_1ms_TL	((65536-(u16)(float)(1000*((float)(MCU_SYSCLK)/(float)(1000000)))) %256)

#define TIMER_12T_1ms_TH	((65536-(u16)(float)(1000*((float)(MCU_SYSCLK)/(float)(12000000)))) /256) 			
#define TIMER_12T_1ms_TL	((65536-(u16)(float)(1000*((float)(MCU_SYSCLK)/(float)(12000000)))) %256)

#define LED_G_0		P25
#define LED_R		P26
#define LED_G_1		P27

#define _LEVEL_HIGH_	1
#define _LEVEL_LOW_		0
#define _DEFAULT_PWM_LEVEL_	_LEVEL_LOW_

idata WordTypeDef wDuty[8];

/*************************************************
*Function:     	void DelayXus(u16 xUs)
*Description:   	delay,unit:us
*Input:     		u8 Us -> *1us  (1~255)
*Output:     
*************************************************/
void DelayXus(u8 xUs)
{
	while(xUs!=0)
	{
#if (MCU_CPUCLK>=11059200)
		_nop_();
#endif
#if (MCU_CPUCLK>=14745600)
		_nop_();
		_nop_();
		_nop_();
		_nop_();
#endif
#if (MCU_CPUCLK>=16000000)
		_nop_();
#endif

#if (MCU_CPUCLK>=22118400)
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
#endif
#if (MCU_CPUCLK>=24000000)
		_nop_();
		_nop_();
#endif		
#if (MCU_CPUCLK>=29491200)
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
		_nop_();
#endif
#if (MCU_CPUCLK>=32000000)
		_nop_();
		_nop_();
#endif

		xUs--;
	}
}

/*************************************************
*Function:     	void DelayXms(u16 xMs)
*Description:    delay,unit:ms
*Input:     		u16 xMs -> *1ms  (1~65535)
*Output:     
*************************************************/
void DelayXms(u16 xMs)
{
	while(xMs!=0)
	{
		CLRWDT();
		DelayXus(200);
		DelayXus(200);
		DelayXus(200);
		DelayXus(200);
		DelayXus(200);
		xMs--;
		
	}
}

/***********************************************************************************
*Function:   	void InitPort()
*Description:	Initialize IO Port
*Input:   
*Output:   		
*************************************************************************************/
void InitPort(void)
{
#if (_DEFAULT_PWM_LEVEL_==_LEVEL_LOW_)
	P2=B10101000;									// PWM output low level
	P3=B11000111;
	PORT_SetP2PushPull(BIT0|BIT1|BIT2|BIT4|BIT6);	// set P20(PWM6),P21(PWM7),P22(CEX0),P24(CEX2),P26(CEX4) as push-pull for PWM output
	PORT_SetP3PushPull(BIT3|BIT4|BIT5);				// set P33(CEX1),P34(CEX3),P35(CEX5) as push-pull for PWM output
	P2=B10101000;
	P3=B11000111;
#else
	P2=0xFF;										// PWM output high level
	P3=0xFF;
	PORT_SetP2PushPull(BIT0|BIT1|BIT2|BIT4|BIT6);	// set P20(PWM6),P21(PWM7),P22(CEX0),P24(CEX2),P26(CEX4) as push-pull for PWM output
	PORT_SetP3PushPull(BIT3|BIT4|BIT5);				// set P33(CEX1),P34(CEX3),P35(CEX5) as push-pull for PWM output
	P2=0xFF;
	P3=0xFF;
#endif

	P10=1;
	PORT_SetP1OpenDrainPu(BIT0);					// Set P10 as open-drain with pull-high

}

/***********************************************************************************
*Function:   	void InitPCA_PWM(void)
*Description:	Initialize PCA for PWM
*Input:   
*Output:   		
*************************************************************************************/
void InitPCA_PWM(void)
{
	PCA_SetCLOCK_SYSCLK();							// PCAʱΪSysClk = 12MHz
	
	PCA_CH0_SetMode_PWM();
	PCA_CH1_SetMode_PWM();
	PCA_CH2_SetMode_PWM();
	PCA_CH3_SetMode_PWM();
	PCA_CH4_SetMode_PWM();
	PCA_CH5_SetMode_PWM();
	PCA_CH6_SetMode_PWM();
	PCA_CH7_SetMode_PWM();

	PCA_CH0_SetPWM_8Bit();
	PCA_CH1_SetPWM_8Bit();
	PCA_CH2_SetPWM_8Bit();
	PCA_CH3_SetPWM_8Bit();
	PCA_CH4_SetPWM_8Bit();
	PCA_CH5_SetPWM_8Bit();
	PCA_CH6_SetPWM_8Bit();
	PCA_CH7_SetPWM_8Bit();

#if (_DEFAULT_PWM_LEVEL_==_LEVEL_LOW_)
	PCA_CH0_SetPWM_EnInvert();	
	PCA_CH1_SetPWM_EnInvert();	
	PCA_CH2_SetPWM_EnInvert();	
	PCA_CH3_SetPWM_EnInvert();	
	PCA_CH4_SetPWM_EnInvert();	
	PCA_CH5_SetPWM_EnInvert();	
	PCA_CH6_SetPWM_EnInvert();	
	PCA_CH7_SetPWM_EnInvert();	
#else
	PCA_CH0_SetPWM_DisInvert();	
	PCA_CH1_SetPWM_DisInvert();	
	PCA_CH2_SetPWM_DisInvert();	
	PCA_CH3_SetPWM_DisInvert();	
	PCA_CH4_SetPWM_DisInvert();	
	PCA_CH5_SetPWM_DisInvert();	
	PCA_CH6_SetPWM_DisInvert();	
	PCA_CH7_SetPWM_DisInvert();	
#endif

	PCA_SetPWM_EdgeAligned();			// Edge

	// Set PWM duty
	PCA_CH0_SetValue_L(0); 				
	PCA_CH0_SetValue_9thBit_L_0();		// 9th bit
	PCA_CH0_SetValue_H(0);				// Relaod value
	PCA_CH0_SetValue_9thBit_H_0();		// 9th bit

	PCA_CH1_SetValue_L(0); 				
	PCA_CH1_SetValue_9thBit_L_0();		// 9th bit
	PCA_CH1_SetValue_H(0);				// Relaod value
	PCA_CH1_SetValue_9thBit_H_0();		// 9th bit

	PCA_CH2_SetValue_L(0); 				
	PCA_CH2_SetValue_9thBit_L_0();		// 9th bit
	PCA_CH2_SetValue_H(0);				// Relaod value
	PCA_CH2_SetValue_9thBit_H_0();		// 9th bit

	PCA_CH3_SetValue_L(0); 				
	PCA_CH3_SetValue_9thBit_L_0();		// 9th bit
	PCA_CH3_SetValue_H(0);				// Relaod value
	PCA_CH3_SetValue_9thBit_H_0();		// 9th bit

	PCA_CH4_SetValue_L(0); 				
	PCA_CH4_SetValue_9thBit_L_0();		// 9th bit
	PCA_CH4_SetValue_H(0);				// Relaod value
	PCA_CH4_SetValue_9thBit_H_0();		// 9th bit

	PCA_CH5_SetValue_L(0); 				
	PCA_CH5_SetValue_9thBit_L_0();		// 9th bit
	PCA_CH5_SetValue_H(0);				// Relaod value
	PCA_CH5_SetValue_9thBit_H_0();		// 9th bit

	PCA_CH6_SetValue_L(0); 				
	PCA_CH6_SetValue_9thBit_L_0();		// 9th bit
	PCA_CH6_SetValue_H(0);				// Relaod value
	PCA_CH6_SetValue_9thBit_H_0();		// 9th bit
	
	PCA_CH7_SetValue_L(0); 				
	PCA_CH7_SetValue_9thBit_L_0();		// 9th bit
	PCA_CH7_SetValue_H(0);				// Relaod value
	PCA_CH7_SetValue_9thBit_H_0();		// 9th bit

	// Set counter
	PCA_SetCounter(0);
	PCA_SetCounterReload(0);

	// Disable PWM Output
	PCA_SetPWM0_DisOutput();					
	PCA_SetPWM1_DisOutput();
	PCA_SetPWM2_DisOutput();					
	PCA_SetPWM3_DisOutput();
	PCA_SetPWM4_DisOutput();					
	PCA_SetPWM5_DisOutput();
	PCA_SetPWM6_DisOutput();					
	PCA_SetPWM7_DisOutput();

	PCA_SetCEX1CEX3CEX5_P33P34P35();	// Set CEX1:P33,CEX3:P34,CEX5:P35

	PCA_EnPCACounter();					// Enable PCA counter
	
}

/***********************************************************************************
*Function:   	void InitClock()
*Description:	Initialize clock
*Input:   
*Output:   		
*************************************************************************************/
void InitClock(void)
{
#if (MCU_SYSCLK==11059200)
#if (MCU_CPUCLK==MCU_SYSCLK)
	// SysClk=11.0592MHz CpuClk=11.0592MHz
	CLK_SetCKCON0(IHRCO_110592MHz|CPUCLK_SYSCLK_DIV_1|SYSCLK_MCKDO_DIV_1);
	
#else
	// SysClk=11.0592MHz CpuClk=5.5296MHz
	CLK_SetCKCON0(IHRCO_110592MHz|CPUCLK_SYSCLK_DIV_2|SYSCLK_MCKDO_DIV_1);
#endif
#endif

#if (MCU_SYSCLK==12000000)
#if (MCU_CPUCLK==MCU_SYSCLK)
	// SysClk=12MHz CpuClk=12MHz
	CLK_SetCKCON0(IHRCO_12MHz|CPUCLK_SYSCLK_DIV_1|SYSCLK_MCKDO_DIV_1);
	
#else
	// SysClk=12MHz CpuClk=6MHz
	CLK_SetCKCON0(IHRCO_12MHz|CPUCLK_SYSCLK_DIV_2|SYSCLK_MCKDO_DIV_1);
#endif
#endif

#if (MCU_SYSCLK==22118400)
#if (MCU_CPUCLK==MCU_SYSCLK)
	// SysClk=22.1184MHz CpuClk=22.1184MHz
	CLK_SetCKCON0(IHRCO_110592MHz|CPUCLK_SYSCLK_DIV_1|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx4, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X4|OSCIn_IHRCO);
#else
	// SysClk=22.1184MHz CpuClk=11.0592MHz
	CLK_SetCKCON0(IHRCO_110592MHz|CPUCLK_SYSCLK_DIV_2|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx4, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X4|OSCIn_IHRCO);
#endif
#endif

#if (MCU_SYSCLK==24000000)
#if (MCU_CPUCLK==MCU_SYSCLK)
	// SysClk=24MHz CpuClk=24MHz
	CLK_SetCKCON0(IHRCO_12MHz|CPUCLK_SYSCLK_DIV_1|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx4, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X4|OSCIn_IHRCO);
#else
	// SysClk=24MHz CpuClk=12MHz
	CLK_SetCKCON0(IHRCO_12MHz|CPUCLK_SYSCLK_DIV_2|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx4, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X4|OSCIn_IHRCO);
#endif
#endif

#if (MCU_SYSCLK==29491200)
#if (MCU_CPUCLK==MCU_SYSCLK)
	// Cpuclk high speed
	CLK_SetCpuCLK_HighSpeed();
	// SysClk=29.491200MHz CpuClk=29.491200MHz
	CLK_SetCKCON0(IHRCO_110592MHz|CPUCLK_SYSCLK_DIV_1|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx5.33, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X533|OSCIn_IHRCO);
#else
	// SysClk=29.491200MHz CpuClk=14.7456MHz
	CLK_SetCKCON0(IHRCO_110592MHz|CPUCLK_SYSCLK_DIV_2|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx5.33, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X533|OSCIn_IHRCO);
#endif
#endif

#if (MCU_SYSCLK==32000000)
#if (MCU_CPUCLK==MCU_SYSCLK)
	// Cpuclk high speed
	CLK_SetCpuCLK_HighSpeed();
	// SysClk=32MHz CpuClk=32MHz
	CLK_SetCKCON0(IHRCO_12MHz|CPUCLK_SYSCLK_DIV_1|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx5.33, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X533|OSCIn_IHRCO);
#else
	// SysClk=32MHz CpuClk=16MHz
	CLK_SetCKCON0(IHRCO_12MHz|CPUCLK_SYSCLK_DIV_2|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx5.33, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X533|OSCIn_IHRCO);
#endif
#endif

#if (MCU_SYSCLK==44236800)
	// SysClk=44.2368MHz CpuClk=22.1184MHz
	CLK_SetCKCON0(IHRCO_110592MHz|CPUCLK_SYSCLK_DIV_1|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx8, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X8|OSCIn_IHRCO);
#endif

#if (MCU_SYSCLK==48000000)
	// SysClk=48MHz CpuClk=24MHz
	CLK_SetCKCON0(IHRCO_12MHz|CPUCLK_SYSCLK_DIV_2|SYSCLK_MCKDO_DIV_1|ENABLE_CKM|CKM_OSCIN_DIV_2);
	DelayXus(100);
	// IHRCO, MCK=CKMIx8, OSCin=IHRCO
	CLK_SetCKCON2(ENABLE_IHRCO|MCK_CKMI_X8|OSCIn_IHRCO);
#endif

	// P60 output MCK/4
	//CLK_P60OC_MCKDiv4();
}

/***********************************************************************************
*Function:       void InitSystem(void)
*Description:    Initialize MCU
*Input:   
*Output:     
*************************************************************************************/
void InitSystem(void)
{
	InitClock();
	InitPort();
	InitPCA_PWM();

}

void main()
{
	u8 cnt,inx;
	bit bOutFlag;
    InitSystem();

	LED_G_0=0;LED_G_0=0;
	DelayXms(1000);
	LED_G_0=1;LED_G_1=1;

	INT_EnAll();
	

	wDuty[0].W=0x0000;
	wDuty[1].W=0x0000;
	wDuty[2].W=0x0000;
	wDuty[3].W=0x0000;
	wDuty[4].W=0x0000;
	wDuty[5].W=0x0000;
	wDuty[6].W=0x0000;
	wDuty[7].W=0x0000;
	bOutFlag=FALSE;
	
	while(1)
    {
    	DelayXms(1);
    	cnt++;
    	if(cnt>=200)
    	{
			cnt=0;
	    	LED_G_1=!LED_G_1;
	    	if(bOutFlag==TRUE)
			{
		   		inx++;
		    	if(inx>=8)
				{
					inx=0;
				}
		    	switch(inx)
				{
					case 0:
						// Duty 100% All High	
						// Set PWM Duty reload value
						wDuty[0].W=0x0000;
						wDuty[1].W=0x0000;
						wDuty[2].W=0x0000;
						wDuty[3].W=0x0000;
						wDuty[4].W=0x0000;
						wDuty[5].W=0x0000;
						wDuty[6].W=0x0000;
						wDuty[7].W=0x0000;
						break;

					case 1:
						// Duty 75% 	
						// Set PWM Duty reload value
						wDuty[0].W=0x0040;
						wDuty[1].W=0x0040;
						wDuty[2].W=0x0040;
						wDuty[3].W=0x0040;
						wDuty[4].W=0x0040;
						wDuty[5].W=0x0040;
						wDuty[6].W=0x0040;
						wDuty[7].W=0x0040;
						break;

					case 2:
						// Duty 50% 	
						// Set PWM Duty reload value
						wDuty[0].W=0x0080;
						wDuty[1].W=0x0080;
						wDuty[2].W=0x0080;
						wDuty[3].W=0x0080;
						wDuty[4].W=0x0080;
						wDuty[5].W=0x0080;
						wDuty[6].W=0x0080;
						wDuty[7].W=0x0080;
						break;

					case 3:
						// Duty 25% 	
						// Set PWM Duty reload value
						wDuty[0].W=0x00C0;
						wDuty[1].W=0x00C0;
						wDuty[2].W=0x00C0;
						wDuty[3].W=0x00C0;
						wDuty[4].W=0x00C0;
						wDuty[5].W=0x00C0;
						wDuty[6].W=0x00C0;
						wDuty[7].W=0x00C0;
						break;
						
					case 4:
						// Duty 0% 	 All Low	
						// Set PWM Duty reload value
						wDuty[0].W=0x0100;
						wDuty[1].W=0x0100;
						wDuty[2].W=0x0100;
						wDuty[3].W=0x0100;
						wDuty[4].W=0x0100;
						wDuty[5].W=0x0100;
						wDuty[6].W=0x0100;
						wDuty[7].W=0x0100;
						break;

					case 5:
						// Duty 25% 	
						// Set PWM Duty reload value
						wDuty[0].W=0x00C0;
						wDuty[1].W=0x00C0;
						wDuty[2].W=0x00C0;
						wDuty[3].W=0x00C0;
						wDuty[4].W=0x00C0;
						wDuty[5].W=0x00C0;
						wDuty[6].W=0x00C0;
						wDuty[7].W=0x00C0;
						break;

					case 6:
						// Duty 50% 	
						// Set PWM Duty reload value
						wDuty[0].W=0x0080;
						wDuty[1].W=0x0080;
						wDuty[2].W=0x0080;
						wDuty[3].W=0x0080;
						wDuty[4].W=0x0080;
						wDuty[5].W=0x0080;
						wDuty[6].W=0x0080;
						wDuty[7].W=0x0080;
						break;

					case 7:
						// Duty 75% 	
						// Set PWM Duty reload value
						wDuty[0].W=0x0040;
						wDuty[1].W=0x0040;
						wDuty[2].W=0x0040;
						wDuty[3].W=0x0040;
						wDuty[4].W=0x0040;
						wDuty[5].W=0x0040;
						wDuty[6].W=0x0040;
						wDuty[7].W=0x0040;
						break;
						
		    	}
				PCA_CH0_SetValue_8Bit(wDuty[0].B.BHigh,wDuty[0].B.BLow);
				PCA_CH1_SetValue_8Bit(wDuty[1].B.BHigh,wDuty[1].B.BLow);
				PCA_CH2_SetValue_8Bit(wDuty[2].B.BHigh,wDuty[2].B.BLow);
				PCA_CH3_SetValue_8Bit(wDuty[3].B.BHigh,wDuty[3].B.BLow);
				PCA_CH4_SetValue_8Bit(wDuty[4].B.BHigh,wDuty[4].B.BLow);
				PCA_CH5_SetValue_8Bit(wDuty[5].B.BHigh,wDuty[5].B.BLow);
				PCA_CH6_SetValue_8Bit(wDuty[6].B.BHigh,wDuty[6].B.BLow);
				PCA_CH7_SetValue_8Bit(wDuty[7].B.BHigh,wDuty[7].B.BLow);
	    	}
    	}
    	if(P10==1)
    	{
			if(bOutFlag==FALSE)
			{
				bOutFlag=TRUE;
				LED_G_0=0;
				// Enable PWM output
				PCA_SetPWM0_EnOutput(); 				
				PCA_SetPWM1_EnOutput();
				PCA_SetPWM2_EnOutput(); 				
				PCA_SetPWM3_EnOutput();
				PCA_SetPWM4_EnOutput(); 				
				PCA_SetPWM5_EnOutput();
				PCA_SetPWM6_EnOutput(); 				
				PCA_SetPWM7_EnOutput();
			}
    	}
    	else
    	{
			if(bOutFlag==TRUE)
			{
				bOutFlag=FALSE;
				LED_G_0=1;
				// Disable PWM output
				PCA_SetPWM0_DisOutput();					
				PCA_SetPWM1_DisOutput();
				PCA_SetPWM2_DisOutput();					
				PCA_SetPWM3_DisOutput();
				PCA_SetPWM4_DisOutput();					
				PCA_SetPWM5_DisOutput();
				PCA_SetPWM6_DisOutput();					
				PCA_SetPWM7_DisOutput();
#if (_DEFAULT_PWM_LEVEL_==_LEVEL_LOW_)
				P2=B10101000;									// PWM output low level
				P3=B11000111;
#else
				P2=0xFF;										// PWM output high level
				P3=0xFF;
#endif

			}
    	}
    }
}


